module Main.LSystems (arrowhead, snowflake, hilbert, getLSystem, hexagonalGosper) where

import Main.Types	(LSystem, Command (..))
import Maybe		(fromMaybe)

lSystems :: [(String, LSystem)]
lSystems = [
		 ("arrowhead", arrowhead)
		,("snowflake", snowflake)
		,("hilbert", hilbert)
		,("tree", tree)
		,("triangle", triangle)
		,("hexagonalGosper", hexagonalGosper)
	]

getLSystem :: String -> LSystem
getLSystem name = fromMaybe undefined (lookup name lSystems)

arrowhead :: LSystem
arrowhead x = f x
	where
	f 0			= Go 10
	f (x + 1)	= g x :#: p :#: f x :#: p :#: g x
	g 0			= Go 10
	g (x + 1)	= f x :#: n :#: g x :#: n :#: f x
	n			= Turn 60
	p			= Turn (-60)

snowflake :: LSystem
snowflake x = f x :#: n :#: n :#: f x :#: n :#: n :#: f x :#: n :#: n
	where
	f 0			= Go 10
	f (x + 1)	= f x :#: p :#: f x :#: n :#: n :#: f x :#: p :#: f x
	n			= Turn 60
	p			= Turn (-60)

hilbert :: LSystem
hilbert x = l x
	where
	l 0			= Sit
	l (x + 1)	= p :#: r x :#: f x :#: n :#: l x :#: f x :#: l x :#: n :#: f x :#: r x :#: p
	r 0			= Sit
	r (x + 1)	= n :#: l x :#: f x :#: p :#: r x :#: f x :#: r x :#: p :#: f x :#: l x :#: n
	f 0			= Go 10
	f (x + 1)	= f x
	n			= Turn 90
	p			= Turn (-90)

hexagonalGosper :: LSystem
hexagonalGosper i = x i :#: f i
	where
	f _			= Go 10
	x 0			= Go 10
	x (i + 1)	= x i :#: p :#: y i :#: f i :#: p :#: p :#: y i :#: f i :#: n :#: f i :#: x i :#: n :#: n :#: f i :#: x i :#: f i :#: x i :#: n :#: y i :#: f i :#: p
	y 0			= Go 10
	y (i + 1)	= n :#: f i :#: x i :#: p :#: y i :#: f i :#: y i :#: f i :#: p :#: p :#: y i :#: f i :#: p :#: f i :#: x i :#: n :#: n :#: f i :#: x i :#: n :#: y i
	n			= Turn 60
	p			= Turn (-60)


-- Sample LSystems taken from the original WH/PW LSystem.hs
triangle :: LSystem
triangle x  =  p :#: f x
  where
  f 0      = Go 10
  f (x+1)  = f x :#: p :#: f x :#: n :#: f x :#: n :#: f x :#: p :#: f x
  n        = Turn 90
  p        = Turn (-90)

tree :: LSystem
tree x  =  f x
  where
  f 0      = Go 10
  f (x+1)  = g x :#: Branch (n :#: f x)
                 :#: Branch (p :#: f x)
                 :#: Branch (g x :#: f x)
  g 0      = Go 10
  g (x+1)  = g x :#: g x
  n        = Turn 45
  p        = Turn (-45)
