module Main.Shapes (lambdaL, sigmaU, getShape, getShapes) where

import Main.Types		(Shape, Command (..))
import Main.Utilities	(left, right)
import Maybe			(fromMaybe)

shapes :: [(String, Shape)]
shapes = [
		  ("lambdaL", lambdaL)
		 ,("sigmaU", sigmaU)
	]

getShape :: String -> Shape
getShape name = fromMaybe undefined (lookup name shapes)

getShapes :: [String] -> [Shape]
getShapes = map getShape

{-
	Commands to draw various shapes

	All shapes need to be of the same relative size.  The shape should assume that the
	turtle starts facing "North" in the bottom left	hand corner of the bounding square.
-}
lambdaL :: Shape
lambdaL = (	right  30 :#: Go (6 * s) :#: left  60 :#: Go (3 * s)
		:#: left   60 :#: Go (1 * s) :#: right 90 :#: Go (1 * s)
		:#: right  90 :#: Go (2 * s) :#: right 60 :#: Go (9 * s)
		:#: left   60 :#: Go (1 * s) :#: right 90 :#: Go (1 * s)
		:#: right  90 :#: Go (2 * s) :#: right 60 :#: Go (4 * s)
		:#: left  120 :#: Go (4 * s) :#: right 60 :#: Go (2 * s),
		[0..360])
	where
		s	= 1 -- Allows for relative scaling, if needed

sigmaU :: Shape
sigmaU = (	right  30 :#: Go (4.0 * s) :#: left   60 :#: Go (4.0 * s)
			:#: right 120 :#: Go (5.0 * s) :#: right  90 :#: Go (1.2 * s)
			:#: right 150 :#: Go (0.8 * s) :#: left   60 :#: Go (3.4 * s)
			:#: left  120 :#: Go (3.4 * s) :#: right  60 :#: Go (3.4 * s)
			:#: left  120 :#: Go (3.4 * s) :#: left   60 :#: Go (0.8 * s)
			:#: right 150 :#: Go (1.2 * s) :#: right  90 :#: Go (5.0 * s),
		[0..360])
	where
		s	= 1.3 -- Allows for relative scaling, if needed
