module Main.Types (Angle, LSystem, Shape, Distance, Turtle, Point (..), Line (..), Bounds (..), Command (..), Pen (..)) where

{-
	Some portions of this file, especially the Distance, Turtle, Pen and
	Command types and the Point instances are taken from the original LSystem
	module by Willem Heijltjes & Phil Wadler and originally based on OpenGL,
	GLFW, and the SOE library by Paul Liu and Duncan Coutts, using fragments of
	SOE's accompanying drawing modules.
-}

import Pictures (Angle)

infixr 5 :#:

type LSystem = (Int -> Command)

type Shape = (Command, [Angle])

type Distance = Float

type Turtle = (Pen, Angle, Point Float)

data (Real a) => Point a = CPoint a a
    deriving (Show, Eq, Ord)

data (Real a) => Line a = CLine Pen (Point a) (Point a)
    deriving (Show, Eq, Ord)

data (Real a) => Bounds a = CBounds (Point a) (Point a)
    deriving (Show, Eq, Ord)

data Command = Go Distance
             | Turn Angle
             | Sit
             | Command :#: Command
             | Branch Command
             | GrabPen Pen
	deriving (Eq, Ord, Show)

data Pen = Colour Float Float Float
         | Inkless
	deriving (Eq, Ord, Show)

instance (Real a) => Num (Point a) where
	CPoint x0 y0 + CPoint x1 y1	= CPoint (x0 + x1) (y0 + y1)
	CPoint x0 y0 - CPoint x1 y1	= CPoint (x0 - x1) (y0 - y1)
	CPoint x0 y0 * CPoint x1 y1	= CPoint (x0 * x1) (y0 * y1)
	fromInteger n				= CPoint (fromInteger n) (fromInteger n)
	abs (CPoint x y)			= CPoint (abs x) (abs y)
	signum (CPoint x y)			= CPoint (signum x) (signum y)

instance (Real a, Fractional a) => Fractional (Point a) where
	CPoint x0 y0 / CPoint x1 y1	= CPoint (x0 / x1) (y0 / y1)
	fromRational r				= CPoint (fromRational r) (fromRational r)
