module Main.Utilities (left, right, optimise, white, black, red, green, blue, pointToTuple, originPointF, originPointI, pointExpand, splitBy) where

import Main.Types	(Angle, Command (..), Pen (..), Point (CPoint), Line (CLine))

{-
	A couple of turning helpers.
	In the context of a "turtle", describing a change of direction
		in terms of left and right feels more natural to me
-}
left :: Angle -> Command
left a = Turn a

right :: Angle -> Command
right a = Turn (-a)

optimise :: Command -> Command
optimise a
	| a == b	= b
	| otherwise	= optimise b
	where
		b = optimise' a
		optimise' :: Command -> Command
		optimise' (Sit :#: x)			= optimise x
		optimise' (x :#: Sit)			= optimise x
		optimise' (Go 0 :#: x)			= optimise x
		optimise' (x :#: Go 0)			= optimise x
		optimise' (Turn 0 :#: x)		= optimise x
		optimise' (x :#: Turn 0)		= optimise x
		optimise' (Go x :#: Go y)		= Go (x + y)
		optimise' (Turn x :#: Turn y)	= Turn (x + y)
		optimise' (x :#: y)				= optimise x :#: optimise y
		optimise' x						= x

white, black, red, green, blue :: Pen
white = Colour 1.0 1.0 1.0
black = Colour 0.0 0.0 0.0
red   = Colour 1.0 0.0 0.0
green = Colour 0.0 1.0 0.0
blue  = Colour 0.0 0.0 1.0

pointToTuple :: (Real a) => Point a -> (a, a)
pointToTuple (CPoint x y) = (x, y)

pointExpand :: Point Int -> Point Float
pointExpand (CPoint x y) = CPoint (fromIntegral x) (fromIntegral y)

-- TODO: Is there any way to make this polymorphic?  Typing it just "Point a" doesn't (or didn't) work
originPointF :: Point Float
originPointF = CPoint 0 0

originPointI :: Point Int
originPointI = CPoint 0 0

splitBy :: Char -> String -> [String]
splitBy	c = split' []
	where
		split' [] []		= []
		split' ys []		= [reverse ys]
		split' ys (x:xs)
			| x == c	= reverse ys : split' [] xs
			| otherwise	= split' (x:ys) xs
